@@ -29,19 +29,13 @@ define([
       var normalized_list = list.map(function(_item) {
         return _item.toLowerCase();
       });
-      return normalized_list.indexOf(item.toLowerCase()) != -1;
+      return normalized_list.indexOf(item.toLowerCase()) !== -1;
     };
 
     var includes_extension = function(filepath, extensionslist) {
       return item_in(extension(filepath), extensionslist);
     };
 
-    var json_or_xml_container_mimetype = function(mimetype) {
-      // Match */*+json or */*+xml
-      return (mimetype.substring(mimetype.length - 5) == '+json'
-              || mimetype.substring(mimetype.length - 4) == '+xml');
-    };
-
     function name_sorter(ascending) {
         return (function(a, b) {
             if (type_order[a['type']] < type_order[b['type']]) {
@@ -58,7 +52,7 @@ define([
             }
             return 0;
         });
-    };
+    }
 
     function modified_sorter(ascending) {
         var order = ascending ? 1 : 0;
@@ -68,9 +62,34 @@ define([
         });
     }
 
+    function size_sorter(ascending) {
+        var order = ascending ? 1 : 0;
+        // directories have file size of undefined
+        return (function(a, b) {
+          if (a.size === undefined) {
+             return (ascending) ? -1 : 1;
+          }
+
+          if (b.size === undefined) {
+             return (ascending) ? 1 : -1;
+          }
+
+          if (a.size > b.size) {
+            return (ascending) ? -1 : 1;
+          }
+
+          if (b.size > a.size) {
+            return (ascending) ? 1 : -1;
+          }
+
+          return 0;
+        });
+    }
+
     var sort_functions = {
         'sort-name': name_sorter,
-        'last-modified': modified_sorter
+        'last-modified': modified_sorter,
+        'file-size': size_sorter
     };
 
     var NotebookList = function (selector, options) {
@@ -102,6 +121,7 @@ define([
         this.sessions = {};
         this.base_url = options.base_url || utils.get_body_data("baseUrl");
         this.notebook_path = options.notebook_path || utils.get_body_data("notebookPath");
+        this.initial_notebook_path = this.notebook_path;
         this.contents = options.contents;
         if (this.session_list && this.session_list.events) {
             this.session_list.events.on('sessions_loaded.Dashboard',
@@ -148,11 +168,10 @@ define([
             $('#new-file').click(function(e) {
                 var w = window.open('', IPython._target);
                 that.contents.new_untitled(that.notebook_path || '', {type: 'file', ext: '.txt'}).then(function(data) {
-                    var url = utils.url_path_join(
+                    w.location = utils.url_path_join(
                         that.base_url, 'edit',
                         utils.encode_uri_components(data.path)
                     );
-                    w.location = url;
                 }).catch(function (e) {
                     w.close();
                     dialog.modal({
@@ -230,7 +249,7 @@ define([
                 var sort_on = e.target.id;
 
                 // Clear sort indications in UI
-                $(".sort-action i").removeClass("fa-arrow-up").removeClass("fa-arrow-down")
+                $(".sort-action i").removeClass("fa-arrow-up").removeClass("fa-arrow-down");
 
                 if ((that.sort_id === sort_on) && (that.sort_direction === 1)) {
                     that.sort_list(sort_on, 0);
@@ -260,8 +279,7 @@ define([
         var files;
         if(dropOrForm === 'drop'){
             files = event.originalEvent.dataTransfer.files;
-        } else
-        {
+        } else  {
             files = event.originalEvent.target.files;
         }
 
@@ -281,12 +299,11 @@ define([
             });
         };
 
-        for (var i = 0; i < files.length; i++) {
-            var f = files[i];
+        Array.from(files).forEach(function(f) {
             var name_and_ext = utils.splitext(f.name);
             var file_ext = name_and_ext[1];
 
-            if (f.size > this._max_upload_size_mb * 1024 * 1024) {
+            if (f.size > that._max_upload_size_mb * 1024 * 1024) {
             	var body_msg = i18n.msg.sprintf(i18n.msg._("The file size is %d MB. Do you still want to upload it?"),
             			Math.round(f.size / (1024 * 1024)));
                 dialog.modal({
@@ -320,7 +337,7 @@ define([
                 reader.onload = reader_onload;
                 reader.onerror = reader_onerror;
             }
-        }
+        });
         // Replace the file input form wth a clone of itself. This is required to
         // reset the form. Otherwise, if you upload a file, delete it and try to
         // upload it again, the changed event won't fire.
@@ -358,13 +375,18 @@ define([
         var that = this;
         // Add an event handler browser back and forward events
         window.onpopstate = function(e) {
-            var path = window.history.state ? window.history.state.path : '';
+            var path = (window.history.state && window.history.state.path) ?
+                    window.history.state.path : that.initial_notebook_path;
             that.update_location(path);
         };
         var breadcrumb = $('.breadcrumb');
         breadcrumb.empty();
         var list_item = $('<li/>');
         var root = $('<li/>').append('<a href="/tree"><i class="fa fa-folder"></i></a>').click(function(e) {
+            // Allow the default browser action when the user holds a modifier (e.g., Ctrl-Click)
+            if(e.altKey || e.metaKey || e.shiftKey) {
+                return true;
+            }
             var path = '';
             window.history.pushState({
                 path: path
@@ -383,6 +405,10 @@ define([
                 utils.encode_uri_components(path)
             );
             var crumb = $('<li/>').append('<a href="' + url + '">' + path_part + '</a>').click(function(e) {
+                // Allow the default browser action when the user holds a modifier (e.g., Ctrl-Click)
+                if(e.altKey || e.metaKey || e.shiftKey) {
+                    return true;
+                }
                 window.history.pushState({
                     path: path
                 }, path, url);
@@ -404,6 +430,10 @@ define([
         $('body').attr('data-notebook-path', path);
         // Update the file tree list without reloading the page
         this.load_list();
+        // Update the page title so the browser tab reflects it
+        // Match how the title appears with a trailing slash or
+        // "Home" if the page loads from the server.
+        $('title').text(path ? path+'/' : i18n.msg._("Home"));
     };
 
     /**
@@ -445,7 +475,7 @@ define([
             model = {
                 type: 'directory',
                 name: '..',
-                path: utils.url_path_split(path)[0],
+                path: utils.url_path_split(path)[0]
             };
             this.add_link(model, item);
             offset += 1;
@@ -515,6 +545,11 @@ define([
             .addClass("item_name")
             .appendTo(link);
 
+        $("<span/>")
+            .addClass("file_size")
+            .addClass("pull-right")
+            .appendTo(item);
+
         $("<span/>")
             .addClass("item_modified")
             .addClass("pull-right")
@@ -555,13 +590,13 @@ define([
     NotebookList.icons = {
         directory: 'folder_icon',
         notebook: 'notebook_icon',
-        file: 'file_icon',
+        file: 'file_icon'
     };
 
     NotebookList.uri_prefixes = {
         directory: 'tree',
         notebook: 'notebooks',
-        file: 'edit',
+        file: 'edit'
     };
 
     /**
@@ -626,7 +661,6 @@ define([
         var has_running_notebook = false;
         var has_directory = false;
         var has_file = false;
-        var that = this;
         var checked = 0;
         $('.list_item :checked').each(function(index, item) {
             var parent = $(item).parent().parent();
@@ -810,8 +844,12 @@ define([
             link.attr('target', IPython._target);
         } else {
             // Replace with a click handler that will use the History API to
-            // push a new route without reloading the page
+            // push a new route without reloading the page if the click is
+            // not modified (e.g., Ctrl-Click)
             link.click(function (e) {
+                if(e.altKey || e.metaKey || e.shiftKey) {
+                    return true;
+                }
                 window.history.pushState({
                     path: model.path
                 }, model.path, utils.url_path_join(
@@ -827,6 +865,9 @@ define([
         // Add in the date that the file was last modified
         item.find(".item_modified").text(utils.format_datetime(model.last_modified));
         item.find(".item_modified").attr("title", moment(model.last_modified).format("YYYY-MM-DD HH:mm"));
+
+        var filesize = utils.format_filesize(model.size);
+        item.find(".file_size").text(filesize || '\xA0');
     };
 
 
@@ -873,7 +914,7 @@ define([
             success : function () {
                 that.load_sessions();
             },
-            error : utils.log_ajax_error,
+            error : utils.log_ajax_error
         };
 
         var session = this.sessions[path];
@@ -905,7 +946,7 @@ define([
         	case 'notebook': return i18n.msg._("Enter a new notebook name:");
         	default: return i18n.msg._("Enter a new name:");
         	}
-        }
+        };
         var rename_title = function (type) {
            	switch(type) {
            	case 'file': return i18n.msg._("Rename file");
@@ -913,7 +954,7 @@ define([
            	case 'notebook': return i18n.msg._("Rename notebook");
            	default: return i18n.msg._("Rename");
            	}
-        }
+        };
         var dialog_body = $('<div/>').append(
             $("<p/>").addClass("rename-message")
                 .text(rename_msg(item_type))
@@ -1204,12 +1245,14 @@ define([
     NotebookList.prototype.add_large_file_upload_button = function (file) {
         var that = this;
         var item = that.new_item(0, true);
+        var stop_signal = false;
         item.addClass('new-file');
         that.add_name_input(file.name, item, 'file');
         var cancel_button = $('<button/>').text("Cancel")
             .addClass("btn btn-default btn-xs")
             .click(function (e) {
                 item.remove();
+                stop_signal = true;
                 return false;
             });
 
@@ -1234,7 +1277,7 @@ define([
                     if ($(v).data('name') === filename) { exists = true; return false; }
                     });
                     return exists
-                }
+                };
                 var exists = check_exist();
                 
                 var add_uploading_button = function (f, item) {
@@ -1257,9 +1300,11 @@ define([
                         var offset = 0;
                         var chunk = 0;
                         var chunk_reader = null;
-                        var upload_file = null;
                         
                         var large_reader_onload = function (event) {
+                            if (stop_signal === true) {
+                                return;
+                            }
                             if (event.target.error == null) {
                                 offset += chunk_size;
                                 if (offset >= f.size) {
@@ -1273,7 +1318,6 @@ define([
                                 upload_file(item, chunk);  // Do the upload
                             } else {
                                 console.log("Read error: " + event.target.error);
-                                return;
                             }
                         };
                         var on_error = function (event) {
@@ -1289,7 +1333,7 @@ define([
                                 body : "Failed to read file '" + name + "'",
                                 buttons : {'OK' : { 'class' : 'btn-primary' }}
                             });
-                        }
+                        };
 
                         chunk_reader = function (_offset, _f) {
                             var reader = new FileReader();
@@ -1330,7 +1374,7 @@ define([
                             model.chunk = chunk;
                             model.content = filedata;
                             
-                            var on_success = function (event) {
+                            var on_success = function () {
                                 if (offset < f.size) {
                                     // of to the next chunk
                                     chunk_reader(offset, f);
@@ -1347,7 +1391,7 @@ define([
                                 }
                             };
                             that.contents.save(path, model).then(on_success, on_error);
-                        }
+                        };
 
                         // now let's start the read with the first block
                         chunk_reader(offset, f);
@@ -1382,7 +1426,7 @@ define([
         item.find(".item_buttons").empty()
             .append(upload_button)
             .append(cancel_button);
-    }
+    };
 
     NotebookList.prototype.add_upload_button = function (item) {
         var that = this;
@@ -1444,7 +1488,6 @@ define([
                     model.content = filedata;
                     content_type = 'application/octet-stream';
                 }
-                filedata = item.data('filedata');
 
                 var on_success = function () {
                     item.removeClass('new-file');
